function [ cdata ] = validateCluster(cdata, param)
% check clusters and eliminate trivial clusters
% The output is added to cdata.clusterInfo(:).valid = 0 or 1

nCluster = length(cdata.clusterInfo);
if cdata.bPair
    areaImg1 = size(cdata.view(1).img,1) * size(cdata.view(1).img,2);
    areaImg2 = size(cdata.view(2).img,1) * size(cdata.view(2).img,2);
else
    areaImg1 = size(cdata.view(1).img,1) * size(cdata.view(1).img,2);
    areaImg2 = areaImg1;
end

% traverse all the cluster, and compute the area of matching regions 
for k=1:nCluster    
    % find matching features
    matchIdxInTheCluster = cdata.clusterInfo(k).matchIdx;
    flipInTheCluster = find(cdata.clusterInfo(k).flipOfMatch);
    curMatchList = cell2mat({ cdata.matchInfo(matchIdxInTheCluster).match }');
    idxFeat1 = curMatchList(:,1);    idxFeat2 = curMatchList(:,2);
    if cdata.bPair
        feat1= cdata.view(1).feat(idxFeat1,:);
        feat2= cdata.view(2).feat(idxFeat2,:); 
        feat2(:,1) = feat2(:,1) + size(cdata.view(1).img,2);
    else
        tmpFeat = idxFeat1(flipInTheCluster);
        idxFeat1(flipInTheCluster) = idxFeat2(flipInTheCluster);
        idxFeat2(flipInTheCluster) = tmpFeat;
        feat1= cdata.view(1).feat(idxFeat1,:);
        feat2= cdata.view(1).feat(idxFeat2,:); 
    end
    
    % compute the convexhull area of each cluster
    %set(0,'CurrentFigure',hFigMain)
    if size(unique(feat1(:,1:2),'rows'),1) > 2 && size(unique(feat2(:,1:2),'rows'),1) > 2
        idxHull1 = convhull(feat1(:,1),feat1(:,2));
        hullFeat1 = feat1(idxHull1,:);
        areaHull1 = polyarea(hullFeat1(:,1),hullFeat1(:,2));

        idxHull2 = convhull(feat2(:,1),feat2(:,2));
        hullFeat2 = feat2(idxHull2,:);
        areaHull2 = polyarea(hullFeat2(:,1),hullFeat2(:,2));

        %fprintf('Area = %11.3f, %11.3f   mean SIFT = %11.3f \n',areaHull1, areaHull2, meanSIFT);
        minArea = min(areaHull1/areaImg1,areaHull2/areaImg2);
    else
        hullFeat1 = [];
        hullFeat2 = [];
        areaHull1 = 0;
        areaHull2 = 0;
        minArea = 0;
    end
        
    % save them in clusterInfo
    cdata.clusterInfo(k).hullFeat1 = hullFeat1;
    cdata.clusterInfo(k).hullFeat2 = hullFeat2;
    cdata.clusterInfo(k).areaHull1 = areaHull1;
    cdata.clusterInfo(k).areaHull2 = areaHull2;
    cdata.clusterInfo(k).minArea = minArea;
    
end

% check validity
% sort by cluster sizes
[ temp traversalIdx ] = sort([ cdata.clusterInfo.size ],'descend');
for k=1:nCluster
    % check the area
    cdata.clusterInfo(traversalIdx(k)).valid = ...
        ( cdata.clusterInfo(traversalIdx(k)).minArea > param.thres_area )...
        && (cdata.clusterInfo(traversalIdx(k)).size > param.thres_size);
    
    % check self-overlaps
    if param.bObjectOverlapCheck && ~cdata.bPair && cdata.clusterInfo(traversalIdx(k)).valid
        overlapArea1 = areaintersection(cdata.clusterInfo(traversalIdx(k)).hullFeat1(:,1:2),...
            cdata.clusterInfo(traversalIdx(k)).hullFeat2(:,1:2), 100);
        if overlapArea1 > param.thres_objectOverlap *cdata.clusterInfo(traversalIdx(k)).areaHull1 ||...
                overlapArea1 > param.thres_objectOverlap*cdata.clusterInfo(traversalIdx(k)).areaHull2
             cdata.clusterInfo(traversalIdx(k)).valid = 0;
        end
    end

    % check overlaps with the larger clusters
    if param.bObjectOverlapCheck && cdata.clusterInfo(traversalIdx(k)).valid
        for l=1:k-1
            if cdata.clusterInfo(traversalIdx(l)).valid 
%                 overlapArea1 = 0;
%                 overlapArea2 = 0;
                overlapArea1 = areaintersection(cdata.clusterInfo(traversalIdx(k)).hullFeat1(:,1:2),...
                    cdata.clusterInfo(traversalIdx(l)).hullFeat1(:,1:2), 100);
                overlapArea2 = areaintersection(cdata.clusterInfo(traversalIdx(k)).hullFeat2(:,1:2),...
                    cdata.clusterInfo(traversalIdx(l)).hullFeat2(:,1:2), 100);
                
                if overlapArea1 > param.thres_objectOverlap *cdata.clusterInfo(traversalIdx(k)).areaHull1 &&...
                        overlapArea2 > param.thres_objectOverlap*cdata.clusterInfo(traversalIdx(k)).areaHull2
                     cdata.clusterInfo(traversalIdx(k)).valid = 0;
                     break;
                else
                    %overlapArea1/cdata.clusterInfo(traversalIdx(k)).areaHull1
                    %overlapArea2/cdata.clusterInfo(traversalIdx(k)).areaHull2
                    
                end
            end
            
            % matching regions can be flipped in single images, thus...
            if ~cdata.bPair && cdata.clusterInfo(traversalIdx(l)).valid 
                overlapArea1 = areaintersection(cdata.clusterInfo(traversalIdx(k)).hullFeat1(:,1:2),...
                    cdata.clusterInfo(traversalIdx(l)).hullFeat2(:,1:2), 100);
                overlapArea2 = areaintersection(cdata.clusterInfo(traversalIdx(k)).hullFeat2(:,1:2),...
                    cdata.clusterInfo(traversalIdx(l)).hullFeat1(:,1:2), 100);
                if overlapArea1 > param.thres_objectOverlap *cdata.clusterInfo(traversalIdx(k)).areaHull1 &&...
                        overlapArea2 > param.thres_objectOverlap*cdata.clusterInfo(traversalIdx(k)).areaHull2
                     cdata.clusterInfo(traversalIdx(k)).valid = 0;
                     break;
                end
            end
            
        end
    end
        
end

